<?php

namespace Drupal\travis_build\Plugin\FrontendEnvironment;

use Drupal\build_hooks\Plugin\FrontendEnvironmentBase;
use Drupal\travis_build\TravisBuildManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'TravisCI' frontend environment type.
 *
 * @FrontendEnvironment(
 *  id = "travisci",
 *  label = "Travis CI",
 *  description = "An environment deployed through Travis CI"
 * )
 */
class TravisCiFrontendEnvironment extends FrontendEnvironmentBase implements ContainerFactoryPluginInterface {

  use MessengerTrait;

  /**
   * The Travis build manager service.
   *
   * @var \Drupal\travis_build\TravisBuildManager
   */
  protected $travisBuildManager;

  /**
   * Construct.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\travis_build\TravisBuildManager $travis_build_manager
   *   The Travis build manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TravisBuildManager $travis_build_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->travisBuildManager = $travis_build_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('travis_build.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function frontEndEnvironmentForm($form, FormStateInterface $form_state) {
    $form['repo_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Repo URL'),
      '#description' => $this->t('Github repo URL of the format <code>organization-name/repo-name</code>.'),
      '#default_value' => $this->configuration['repo_url'] ?? '',
      '#required' => TRUE,
    ];

    $form['branch'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Branch or tag'),
      '#description' => $this->t('Enter which branch or tag to trigger a build.'),
      '#default_value' => $this->configuration['branch'] ?? '',
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function frontEndEnvironmentSubmit($form, FormStateInterface $form_state) {
    $this->configuration['repo_url'] = $form_state->getValue('repo_url');
    $this->configuration['branch'] = $form_state->getValue('branch');
  }

  /**
   * {@inheritdoc}
   */
  public function getBuildHookDetails() {
    return $this->travisBuildManager->getBuildHookDetailsForPluginConfiguration($this->getConfiguration());
  }

  /**
   * {@inheritdoc}
   */
  public function getAdditionalDeployFormElements(FormStateInterface $form_state) {
    // This plugin adds to the deployment form a fieldset displaying the
    // latest deployments:
    $form = [];

    $form['latestTravisCiDeployments'] = [
      '#type' => 'details',
      '#title' => $this->t('Recent deployments'),
      '#description' => $this->t('Here you can see the details for the latest deployments for this environment.'),
      '#open' => TRUE,
    ];

    try {
      $form['latestTravisCiDeployments']['table'] = $this->getLastTravisCiDeploymentsTable($this->getConfiguration());

      $form['latestTravisCiDeployments']['refresher'] = [
        '#type' => 'button',
        '#ajax' => [
          'callback' => [TravisCiFrontendEnvironment::class, 'refreshDeploymentTable'],
          'wrapper' => 'ajax-replace-table',
          'effect' => 'fade',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Refreshing deployment status...'),
          ],
        ],
        '#value' => $this->t('Refresh'),
      ];
    }
    catch (GuzzleException $e) {
      $this->messenger()->addError('Unable to retrieve information about the last deployments for this environment. Check configuration.');
    }

    return $form;
  }

  /**
   * Gets info about the latest Travis CI deployments for this environment.
   *
   * @param array $settings
   *   The plugin settings array.
   *
   * @return array
   *   Renderable array.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  private function getLastTravisCiDeploymentsTable(array $settings) {
    $travisCiData = $this->travisBuildManager->retrieveLatestBuildsFromTravisCiForEnvironment($settings, 8);
    $element = [
      '#type' => 'table',
      '#attributes' => ['id' => 'ajax-replace-table'],
      '#header' => [
        $this->t('State'),
        $this->t('Started at'),
        $this->t('Finished at'),
        $this->t('Branch'),
        $this->t('Commit hash'),
        $this->t('Message'),
      ],
    ];

    if (!empty($travisCiData)) {
      foreach ($travisCiData as $build) {
        $element[$build['number']]['state'] = [
          '#type' => 'item',
          '#markup' => '<strong>' . $build['state'] . '</strong>',
        ];

        $element[$build['number']]['started_at'] = [
          '#type' => 'item',
          '#markup' => $build['started_at'],
        ];

        $element[$build['number']]['finished_at'] = [
          '#type' => 'item',
          '#markup' => $build['finished_at'],
        ];

        $element[$build['number']]['branch'] = [
          '#type' => 'item',
          '#markup' => $build['branch']['name'],
        ];

        $element[$build['number']]['commit_sha'] = [
          '#type' => 'item',
          '#markup' => $build['commit']['sha'],
        ];

        $element[$build['number']]['message'] = [
          '#type' => 'item',
          '#markup' => $build['commit']['message'],
        ];
      }
    }
    return $element;
  }

  /**
   * Ajax form callback to rebuild the latest deployments table.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state of the form.
   *
   * @return array
   *   The form array to add back to the form.
   */
  public static function refreshDeploymentTable(array $form, FormStateInterface $form_state) {
    return $form['latestTravisCiDeployments']['table'];
  }

}
