<?php

namespace Drupal\travis_build;

use Drupal\build_hooks\BuildHookDetails;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use GuzzleHttp\ClientInterface;

/**
 * Class TravisBuildManager.
 */
class TravisBuildManager {

  /**
   * The Travis API url, with organization and repo name.
   */
  const API_BASE_URL = 'https://api.travis-ci.com/repo/';

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * GuzzleHttp\ClientInterface definition.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The Date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a new NetlifyManager object.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ClientInterface $http_client, DateFormatterInterface $date_formatter) {
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * Get the request options for the API request.
   *
   * @param array $config
   *   The plugin configuration array.
   *
   * @return array
   *   Array of options to pass to GuzzleHttp request.
   */
  private function getRequestOptions(array $config) {
    $settings = $this->configFactory->get('travis_build.settings');
    $auth_token = $settings->get('auth_token');
    if (!$auth_token) {
      return NULL;
    }

    $options = [
      'verify' => FALSE,
      'headers' => [
        'Content-Type' => 'application/json',
        'Accept' => 'application/json',
        'Travis-API-Version' => '3',
        'Authorization' => 'token ' . $auth_token,
      ],
    ];

    return $options;
  }

  /**
   * Build the url for the Travis frontend.
   *
   * @param array $config
   *   The plugin configuration array.
   * @param string $path
   *   The API path, which will be prefixed with the base url.
   *
   * @return string
   *   The url to call to get the builds.
   */
  private function buildTravisUrlForEnvironment(array $config, string $path = '/builds') {
    $settings = $this->configFactory->get('travis_build.settings');
    // Sanitize the repo url - convert / to %2F and drop trailing slash.
    $repo_url = str_replace('/', '%2F', rtrim($config['repo_url'], '/'));
    return self::API_BASE_URL . $repo_url . $path;
  }

  /**
   * Returns the build hooks details based on plugin configuration.
   *
   * @param array $config
   *   The plugin configuration array.
   *
   * @return \Drupal\build_hooks\BuildHookDetails
   *   Build hooks detail object with info about the request to make.
   *
   * @throws \Exception
   */
  public function getBuildHookDetailsForPluginConfiguration(array $config) {
    $buildHookDetails = new BuildHookDetails();
    $build_url = $this->buildTravisUrlForEnvironment($config, '/requests');
    if (!$build_url) {
      throw new \Exception($this->t('Travis frontend environment is not well configured.'));
    }

    $buildHookDetails->setUrl($build_url);
    $buildHookDetails->setMethod('POST');
    $options = $this->getRequestOptions($config);
    $options['json'] = [
      'branch' => $config['branch'],
    ];
    $buildHookDetails->setBody($options);

    return $buildHookDetails;
  }

  /**
   * Get the latest builds from Travis CI for an environment.
   *
   * @param array $config
   *   The plugin configuration array.
   * @param int $limit
   *   Number of desired builds to retrieve.
   *
   * @return array
   *   An array with info about the builds.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function retrieveLatestBuildsFromTravisCiForEnvironment(array $config, $limit = 8) {
    $url = $this->buildTravisUrlForEnvironment($config, '/builds');

    $options = $this->getRequestOptions($config);
    $options['query'] = [
      'branch.name' => $config['branch'],
      'limit' => $limit,
    ];

    $response = $this->httpClient->request('GET', $url, $options);
    $payload = Json::decode($response->getBody()->getContents(), TRUE);

    if (!empty($payload['builds'])) {
      return $payload['builds'];
    }

    return NULL;
  }

}
